import google.auth
from google.cloud import storage
from configs import GCS_BUCKET_NAME, PIPELINE_NAME, CLUSTER_SCALING
import cluster_creation
import network_creation
import json
import requests
import google.auth.transport.requests
import os
import subprocess
import sys
import time

# Get default project and authentication
try:
    _, GOOGLE_CLOUD_PROJECT = google.auth.default()
except google.auth.exceptions.DefaultCredentialsError:
    GOOGLE_CLOUD_PROJECT = ''

### BUCKET CREATION ###
# Creates bucket with necessary subdirectories
print("Building GCS bucket...")
try:
    client = storage.Client(project=GOOGLE_CLOUD_PROJECT)
    try:
        bucket = storage.Bucket(client, name=GCS_BUCKET_NAME)
        bucket.create(location='eu')
        print("Created bucket {} in project {}.".format(GCS_BUCKET_NAME, GOOGLE_CLOUD_PROJECT))
    except Exception as e:
        if e.code == 409:
            print("Bucket already exists.")
        else:
            raise
    try:
        bucket = client.get_bucket(GCS_BUCKET_NAME)
        path_modules = bucket.blob('input/modules/')
        path_data = bucket.blob('input/data/')
        path_output = bucket.blob('output/')
        path_modules.upload_from_string('')
        path_data.upload_from_string('')
        path_output.upload_from_string('')
        print("Built bucket hierarchy.")
    except Exception as e:
        print(e)
except Exception as e:
    print(e)

# Updloading necessary files to correct subdirectories
try:
    client = storage.Client(project=GOOGLE_CLOUD_PROJECT)
    bucket = client.get_bucket(GCS_BUCKET_NAME)
    preprocessing = bucket.blob('input/modules/preprocessing.py')
    trainer = bucket.blob('input/modules/trainer.py')
    data = bucket.blob('input/data/data.csv')
    preprocessing.upload_from_filename('./modules/preprocessing.py')
    print("Uploaded preprocessing.py to bucket {}/input/modules/preprocessing.py".format(GCS_BUCKET_NAME))
    trainer.upload_from_filename('./modules/trainer.py')
    print("Uploaded trainer.py to bucket {}/input/modules/trainer.py".format(GCS_BUCKET_NAME))
    data.upload_from_filename('./data/data.csv')
    print("Uploaded data to bucket {}/input/data/data.csv".format(GCS_BUCKET_NAME))
except Exception as e:
    print(e)

### Network creation ###
POST_URL = "https://www.googleapis.com/compute/v1/projects/{}/global/networks".format(GOOGLE_CLOUD_PROJECT)
POST_PAYLOAD = network_creation.network
POST_PAYLOAD["selfLink"] = "projects/{}/global/networks/default".format(GOOGLE_CLOUD_PROJECT)
POST_PAYLOAD = json.dumps(POST_PAYLOAD)

# Creating credential token
creds, project = google.auth.default(scopes=["https://www.googleapis.com/auth/cloud-platform"])
auth_req = google.auth.transport.requests.Request()
creds.refresh(auth_req)

# REST API call to create network
print("Building network...")
response = requests.post(POST_URL, data = POST_PAYLOAD, headers={'Authorization': 'Bearer {}'.format(creds.token)})
resp_json = json.loads(response.text)

# Catching HTTP errors
if response.status_code != 200:
    if resp_json["error"]["errors"][0]["reason"] == "alreadyExists":
        print("Network already exists.")
    else:
        print("Network creation failed. \n")
        print(response.status_code)
        print(response.text)
else:
    print("Network is deploying...")

### CLUSTER CREATION ###
POST_PAYLOAD = {}
POST_URL = ""

# Defines which JSON data to POST to which URL
if CLUSTER_SCALING == 'standard':
    POST_PAYLOAD = cluster_creation.standard
    POST_URL = "https://container.googleapis.com/v1beta1/projects/{}/zones/europe-west3-a/clusters".format(GOOGLE_CLOUD_PROJECT)
else:
    POST_PAYLOAD = cluster_creation.autopilot
    POST_URL = "https://container.googleapis.com/v1/projects/{}/locations/europe-west3/clusters".format(GOOGLE_CLOUD_PROJECT)

# Set variables for cluster creation
CLUSTER_NAME = PIPELINE_NAME + '-cluster'
CLUSTER_NAME = CLUSTER_NAME.replace('_','-')

# Alters payload to match GCP project and adds relevant cluster name
POST_PAYLOAD["cluster"]["name"] = CLUSTER_NAME
POST_PAYLOAD["cluster"]["network"] = "projects/{}/global/networks/default".format(GOOGLE_CLOUD_PROJECT)
POST_PAYLOAD["cluster"]["subnetwork"] = "projects/{}/regions/europe-west3/subnetworks/default".format(GOOGLE_CLOUD_PROJECT)
# Dict to JSON for POST data type
POST_PAYLOAD = json.dumps(POST_PAYLOAD)

# Creating credential token
creds, project = google.auth.default(scopes=["https://www.googleapis.com/auth/cloud-platform"])
auth_req = google.auth.transport.requests.Request()
creds.refresh(auth_req)

# REST API call to build cluster
print("Building cluster...")
response = requests.post(POST_URL, data = POST_PAYLOAD, headers={'Authorization': 'Bearer {}'.format(creds.token)})
resp_json = json.loads(response.text)

# Waiting for network to be fully configured
while response.status_code == 400:
    response = requests.post(POST_URL, data = POST_PAYLOAD, headers={'Authorization': 'Bearer {}'.format(creds.token)})
    resp_json = json.loads(response.text)

# Catching HTTP errors when creating cluster
if response.status_code != 200:
    if resp_json["error"]["status"] == "ALREADY_EXISTS":
        print("Cluster already exists.")
    else:
        print("Cluster build failed. \n")
        print(response.status_code)
        print(response.text)
else:
    print("Cluster build completed successfully. Cluster deployment in progress. Continue with KFP deployment.")
